program DecimalsDevelop;

{$APPTYPE CONSOLE}
{ $DEFINE NOCONSOLE}

// Be sure to set console buffer to 160 char width in Properties of console window.

uses
  SysUtils,
  Math,
  Console,
  Decimals in 'Decimals.pas';

{$IFNDEF NOCONSOLE}
var
  LineVal: Integer = 0;
{$ENDIF}

procedure SetLineColour;
begin
{$IFNDEF NOCONSOLE}
  if Odd(LineVal) then
    TextColor(LightGreen)
  else
    TextColor(White);
  Inc(LineVal);
{$ENDIF}
end;

procedure WriteDecimal(const D: Decimal);
var
  A: TLongwordArray;
const
  Signs: array[0..1] of Char = ('+', '-');
begin
  A := Decimal.GetBits(D);
  Writeln(Format('%36s: %s %.2X %.8X %.8X %.8X',
    [string(D), Signs[A[3] shr 31], A[3] shr 16 and $FF, A[2], A[1], A[0]]));
end;

procedure WriteSD(const s: string; D: Decimal);
begin
  SetLineColour;
  Write(s);
  WriteDecimal(D);
end;

procedure Banner(const S: string);
{$IFNDEF NOCONSOLE}
var
  BGColor: Byte;
  WindowWidth: Integer;
{$ENDIF}
begin
{$IFNDEF NOCONSOLE}
  WindowWidth := Lo(Console.WindMax);
  BGColor := Console.TextAttr;
  Console.TextColor(Yellow);
{$ENDIF}
  Writeln;
{$IFNDEF NOCONSOLE}
  GotoXY((WindowWidth - Length(S)) div 2, Console.WhereY);
{$ELSE}
  Write(StringOfChar(' ', 80-Length(S) div 2));
{$ENDIF}
  Writeln(S);
{$IFNDEF NOCONSOLE}
  GotoXY((WindowWidth - Length(S)) div 2, Console.WhereY);
{$ELSE}
  Write(StringOfChar(' ', 80-Length(S) div 2));
{$ENDIF}
  Writeln(StringOfChar('-', Length(S)));
  Writeln;
{$IFNDEF NOCONSOLE}
  Console.TextColor(BGColor);
{$ENDIF}
end;

procedure WriteLine(const F: string; params: array of const);
begin
  SetLineColour;
  Writeln(Format(F, params));
end;

procedure FromFloat(E: Extended);
var
  D: Decimal;
  S: Single;
  Db: Double;
begin
  D := 0;
  S := E;
  Db := E;
  WriteLine('Extended:     %39.18g', [E]);
  try
    D := E;
    WriteSD('D from Extended: ', D);
  except
    on E: Exception do
      WriteLine('%s: %s', [E.ClassName, E.Message]);
  end;
  D := 0;
  WriteLine('Double:       %39.18g', [Db]);
  try
    D := Db;
    WriteSD('D from Double:   ', D);
  except
    on E: Exception do
      WriteLine('%s: %s', [E.ClassName, E.Message]);
  end;
  D := 0;
  WriteLine('Single:       %39.18g', [S]);
  try
    D := S;
    WriteSD('D from Single:   ', D);
  except
    on E: Exception do
      WriteLine('%s: %s', [E.ClassName, E.Message]);
  end;
  Writeln;
end;

procedure FromFloats(Str: string);
var
  D: Decimal;
  S: Single;
  Db: Double;
  E: Extended;
begin
  WriteLine('string:       ''%s''', [Str]);
  S := StrToFloat(Str);
  D := S;
  WriteLine('Single:       %39.18g', [S]);
  WriteSD('D from Single:   ', D);
  Db := StrToFloat(Str);
  D := Db;
  WriteLine('Double:       %39.18g', [Db]);
  WriteSD('D from Double:   ', D);
  E := StrToFloat(Str);
  D := E;
  WriteLine('Extended:     %39.18g', [E]);
  WriteSD('D from Extended: ', E);
  WriteSD('D from string:   ', Str);
  Writeln;
end;

procedure Test10;
var
  I: Integer;
begin
  Banner('Conversions to Decimal');
  FromFloats('0,1');
  FromFloats('0,2');
  FromFloats('0,5');
  FromFloats('0,7');
  FromFloats('0,75');
  FromFloats('0');
  FromFloats('0,0');
  FromFloats('0,00000000000');
  FromFloats('1E-28');
  FromFloats(Decimal.Pi.ToString);
  FromFloats('1e20');
  for I := 1 to 10 do
    FromFloats(Format('%30.18g', [100 * (2 * Random - 1.0)]));
  Banner('Conversions from special values');
  FromFloat(Math.NaN);
  FromFloat(Math.Infinity);
  FromFloat(Math.NegInfinity);
end;

procedure Test11;
var
  D: Decimal;
begin
  Banner('Conversions');
  D := '1234567.891711124567837564';
  WriteSD('Decimal:     ', D);
  WriteLine('Extended: %39.18g', [D.ToExtended]);
  WriteLine('Currency: %39.18g', [D.ToCurrency]);
  WriteLine('Longint:  %39d (%0:.8X)', [D.ToLongint]);
  WriteLine('Longword: %39d (%0:.8X)', [D.ToLongword]);
  WriteLine('Int64:    %39d (%0:.16X)', [D.ToInt64]);
  WriteLine('UInt64:   %39d (%0:.16X)', [D.ToUint64]);
  WriteSD('Decimal:     ', D);
  Writeln;
  D := '1234567.3917111245678';
  WriteSD('Decimal:     ', D);
  WriteLine('Extended: %39.18g', [D.ToExtended]);
  WriteLine('Currency: %39.18g', [D.ToCurrency]);
  WriteLine('Longint:  %39d (%0:.8X)', [D.ToLongint]);
  WriteLine('Longword: %39d (%0:.8X)', [D.ToLongword]);
  WriteLine('Int64:    %39d (%0:.16X)', [D.ToInt64]);
  WriteLine('UInt64:   %39d (%0:.16X)', [D.ToUint64]);
  WriteSD('Decimal:     ', D);
  Writeln;
  D := '123456739171.11245678';
  WriteSD('Decimal:     ', D);
  WriteLine('Extended: %39.18g', [D.ToExtended]);
  WriteLine('Currency: %39.18g', [D.ToCurrency]);
  try
    WriteLine('Longint:  %39d (%0:.8X)', [D.ToLongint]);
  except on E: Exception do
    Writeln(E.ClassName, ': ', E.Message);
  end;
  try
    WriteLine('Longword: %39d (%0:.8X)', [D.ToLongword]);
  except on E: Exception do
    Writeln(E.ClassName, ': ', E.Message);
  end;
  WriteLine('Int64:    %39d (%0:.16X)', [D.ToInt64]);
  WriteLine('UInt64:   %39d (%0:.16X)', [D.ToUint64]);
  WriteSD('Decimal:     ', D);
  Writeln;
  Writeln('SizeOf(Decimal) = ', SizeOf(Decimal), ' (', SizeOf(Decimal) * 8, ' bits)');
end;

begin
{$IFNDEF NOCONSOLE}
  TextBackground(Black);
  ClrScr;
{$ENDIF}
//  Randomize;
  try
    Test10;
    Test11;
  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
{$IFNDEF NOCONSOLE}
  ReadKey;
{$ELSE}
  Readln;
{$ENDIF}
end.

